@@ -5,22 +5,26 @@ import (
 	"reflect"
 	"time"
 
-	"github.com/golang/glog"
-
 	"k8s.io/apimachinery/pkg/api/errors"
 	kmeta "k8s.io/apimachinery/pkg/api/meta"
 	metaapi "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/wait"
 	kubeinformers "k8s.io/client-go/informers"
 	kubeset "k8s.io/client-go/kubernetes"
+	appsset "k8s.io/client-go/kubernetes/typed/apps/v1"
+	coreset "k8s.io/client-go/kubernetes/typed/core/v1"
+	rbacset "k8s.io/client-go/kubernetes/typed/rbac/v1"
 	restclient "k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/util/workqueue"
+	"k8s.io/klog"
 
 	operatorapi "github.com/openshift/api/operator/v1"
 	configset "github.com/openshift/client-go/config/clientset/versioned"
+	configsetv1 "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
 	configinformers "github.com/openshift/client-go/config/informers/externalversions"
 	routeset "github.com/openshift/client-go/route/clientset/versioned"
+	routesetv1 "github.com/openshift/client-go/route/clientset/versioned/typed/route/v1"
 	routeinformers "github.com/openshift/client-go/route/informers/externalversions"
 
 	configapiv1 "github.com/openshift/api/config/v1"
@@ -33,12 +37,11 @@ import (
 	"github.com/openshift/cluster-image-registry-operator/pkg/resource/object"
 	"github.com/openshift/cluster-image-registry-operator/pkg/resource/strategy"
 	"github.com/openshift/cluster-image-registry-operator/pkg/storage"
-	"github.com/openshift/cluster-image-registry-operator/pkg/util"
 )
 
 const (
 	openshiftConfigNamespace = "openshift-config"
-	installerConfigNamespace = "kube-system"
+	kubeSystemNamespace      = "kube-system"
 	workqueueKey             = "changes"
 	defaultResyncDuration    = 10 * time.Minute
 )
@@ -59,10 +62,14 @@ func (e permanentError) Error() string {
 	return e.Err.Error()
 }
 
+// NewController returns a controller for openshift image registry objects.
+//
+// This controller keeps track of resources needed in order to have openshift
+// internal registry working.
 func NewController(kubeconfig *restclient.Config) (*Controller, error) {
 	namespace, err := regopclient.GetWatchNamespace()
 	if err != nil {
-		glog.Fatalf("failed to get watch namespace: %s", err)
+		klog.Fatalf("failed to get watch namespace: %s", err)
 	}
 
 	p := parameters.Globals{}
@@ -80,14 +87,17 @@ func NewController(kubeconfig *restclient.Config) (*Controller, error) {
 	p.ImageConfig.Name = "cluster"
 	p.CAConfig.Name = imageregistryv1.ImageRegistryCertificatesName
 	p.ServiceCA.Name = "serviceca"
+	p.TrustedCA.Name = "trusted-ca"
 
 	listers := &regopclient.Listers{}
+	clients := &regopclient.Clients{}
 	c := &Controller{
 		kubeconfig: kubeconfig,
 		params:     p,
-		generator:  resource.NewGenerator(kubeconfig, listers, &p),
+		generator:  resource.NewGenerator(kubeconfig, clients, listers, &p),
 		workqueue:  workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "Changes"),
 		listers:    listers,
+		clients:    clients,
 	}
 
 	// Initial event to bootstrap CR if it doesn't exist.
@@ -96,12 +106,14 @@ func NewController(kubeconfig *restclient.Config) (*Controller, error) {
 	return c, nil
 }
 
+// Controller keeps track of openshift image registry components.
 type Controller struct {
 	kubeconfig *restclient.Config
 	params     parameters.Globals
 	generator  *resource.Generator
 	workqueue  workqueue.RateLimitingInterface
 	listers    *regopclient.Listers
+	clients    *regopclient.Clients
 }
 
 func (c *Controller) createOrUpdateResources(cr *imageregistryv1.Config) error {
@@ -137,7 +149,7 @@ func (c *Controller) sync() error {
 		err = c.finalizeResources(cr)
 
 		if genErr := c.generator.ApplyClusterOperator(cr); genErr != nil {
-			glog.Errorf("unable to apply cluster operator: %s", genErr)
+			klog.Errorf("unable to apply cluster operator: %s", genErr)
 		}
 
 		return err
@@ -154,7 +166,7 @@ func (c *Controller) sync() error {
 	case operatorapi.Unmanaged:
 		// ignore
 	default:
-		glog.Warningf("unknown custom resource state: %s", cr.Spec.ManagementState)
+		klog.Warningf("unknown custom resource state: %s", cr.Spec.ManagementState)
 	}
 
 	deploy, err := c.listers.Deployments.Get(imageregistryv1.ImageRegistryName)
@@ -173,23 +185,18 @@ func (c *Controller) sync() error {
 	if metadataChanged || specChanged {
 		difference, err := object.DiffString(prevCR, cr)
 		if err != nil {
-			glog.Errorf("unable to calculate difference in %s: %s", util.ObjectInfo(cr), err)
+			klog.Errorf("unable to calculate difference in %s: %s", utilObjectInfo(cr), err)
 		}
-		glog.Infof("object changed: %s (metadata=%t, spec=%t): %s", util.ObjectInfo(cr), metadataChanged, specChanged, difference)
+		klog.Infof("object changed: %s (metadata=%t, spec=%t): %s", utilObjectInfo(cr), metadataChanged, specChanged, difference)
 
 		if genErr := c.generator.ApplyClusterOperator(cr); genErr != nil {
-			glog.Errorf("unable to apply cluster operator: %s", genErr)
+			klog.Errorf("unable to apply cluster operator: %s", genErr)
 		}
 
-		client, err := regopset.NewForConfig(c.kubeconfig)
-		if err != nil {
-			return err
-		}
-
-		updatedCR, err := client.ImageregistryV1().Configs().Update(cr)
+		updatedCR, err := c.clients.RegOp.ImageregistryV1().Configs().Update(cr)
 		if err != nil {
 			if !errors.IsConflict(err) {
-				glog.Errorf("unable to update %s: %s", util.ObjectInfo(cr), err)
+				klog.Errorf("unable to update %s: %s", utilObjectInfo(cr), err)
 			}
 			return err
 		}
@@ -204,23 +211,18 @@ func (c *Controller) sync() error {
 	if statusChanged {
 		difference, err := object.DiffString(prevCR, cr)
 		if err != nil {
-			glog.Errorf("unable to calculate difference in %s: %s", util.ObjectInfo(cr), err)
+			klog.Errorf("unable to calculate difference in %s: %s", utilObjectInfo(cr), err)
 		}
-		glog.Infof("object changed: %s (status=%t): %s", util.ObjectInfo(cr), statusChanged, difference)
+		klog.Infof("object changed: %s (status=%t): %s", utilObjectInfo(cr), statusChanged, difference)
 
 		if genErr := c.generator.ApplyClusterOperator(cr); genErr != nil {
-			glog.Errorf("unable to apply cluster operator (cr status=%t): %s", statusChanged, genErr)
+			klog.Errorf("unable to apply cluster operator (cr status=%t): %s", statusChanged, genErr)
 		}
 
-		client, err := regopset.NewForConfig(c.kubeconfig)
-		if err != nil {
-			return err
-		}
-
-		_, err = client.ImageregistryV1().Configs().UpdateStatus(cr)
+		_, err = c.clients.RegOp.ImageregistryV1().Configs().UpdateStatus(cr)
 		if err != nil {
 			if !errors.IsConflict(err) {
-				glog.Errorf("unable to update status %s: %s", util.ObjectInfo(cr), err)
+				klog.Errorf("unable to update status %s: %s", utilObjectInfo(cr), err)
 			}
 			return err
 		}
@@ -240,22 +242,22 @@ func (c *Controller) eventProcessor() {
 			return
 		}
 
-		glog.V(1).Infof("get event from workqueue")
+		klog.V(1).Infof("get event from workqueue")
 		func() {
 			defer c.workqueue.Done(obj)
 
 			if _, ok := obj.(string); !ok {
 				c.workqueue.Forget(obj)
-				glog.Errorf("expected string in workqueue but got %#v", obj)
+				klog.Errorf("expected string in workqueue but got %#v", obj)
 				return
 			}
 
 			if err := c.sync(); err != nil {
 				c.workqueue.AddRateLimited(workqueueKey)
-				glog.Errorf("unable to sync: %s, requeuing", err)
+				klog.Errorf("unable to sync: %s, requeuing", err)
 			} else {
 				c.workqueue.Forget(obj)
-				glog.Infof("event from workqueue successfully processed")
+				klog.Infof("event from workqueue successfully processed")
 			}
 		}()
 	}
@@ -269,18 +271,18 @@ func (c *Controller) handler() cache.ResourceEventHandlerFuncs {
 					return
 				}
 			}
-			glog.V(1).Infof("add event to workqueue due to %s (add)", util.ObjectInfo(o))
+			klog.V(1).Infof("add event to workqueue due to %s (add)", utilObjectInfo(o))
 			c.workqueue.Add(workqueueKey)
 		},
 		UpdateFunc: func(o, n interface{}) {
 			newAccessor, err := kmeta.Accessor(n)
 			if err != nil {
-				glog.Errorf("unable to get accessor for new object: %s", err)
+				klog.Errorf("unable to get accessor for new object: %s", err)
 				return
 			}
 			oldAccessor, err := kmeta.Accessor(o)
 			if err != nil {
-				glog.Errorf("unable to get accessor for old object: %s", err)
+				klog.Errorf("unable to get accessor for old object: %s", err)
 				return
 			}
 			if newAccessor.GetResourceVersion() == oldAccessor.GetResourceVersion() {
@@ -293,7 +295,7 @@ func (c *Controller) handler() cache.ResourceEventHandlerFuncs {
 					return
 				}
 			}
-			glog.V(1).Infof("add event to workqueue due to %s (update)", util.ObjectInfo(n))
+			klog.V(1).Infof("add event to workqueue due to %s (update)", utilObjectInfo(n))
 			c.workqueue.Add(workqueueKey)
 		},
 		DeleteFunc: func(o interface{}) {
@@ -301,31 +303,64 @@ func (c *Controller) handler() cache.ResourceEventHandlerFuncs {
 			if !ok {
 				tombstone, ok := o.(cache.DeletedFinalStateUnknown)
 				if !ok {
-					glog.Errorf("error decoding object, invalid type")
+					klog.Errorf("error decoding object, invalid type")
 					return
 				}
 				object, ok = tombstone.Obj.(metaapi.Object)
 				if !ok {
-					glog.Errorf("error decoding object tombstone, invalid type")
+					klog.Errorf("error decoding object tombstone, invalid type")
 					return
 				}
-				glog.V(4).Infof("recovered deleted object %q from tombstone", object.GetName())
+				klog.V(4).Infof("recovered deleted object %q from tombstone", object.GetName())
 			}
 			if clusterOperator, ok := o.(*configapiv1.ClusterOperator); ok {
 				if clusterOperator.GetName() != imageregistryv1.ImageRegistryClusterOperatorResourceName {
 					return
 				}
 			}
-			glog.V(1).Infof("add event to workqueue due to %s (delete)", util.ObjectInfo(object))
+			klog.V(1).Infof("add event to workqueue due to %s (delete)", utilObjectInfo(object))
 			c.workqueue.Add(workqueueKey)
 		},
 	}
 }
 
+// Run starts the Controller.
 func (c *Controller) Run(stopCh <-chan struct{}) error {
 	defer c.workqueue.ShutDown()
 
-	kubeClient, err := kubeset.NewForConfig(c.kubeconfig)
+	var err error
+
+	c.clients.Core, err = coreset.NewForConfig(c.kubeconfig)
+	if err != nil {
+		return err
+	}
+
+	c.clients.Apps, err = appsset.NewForConfig(c.kubeconfig)
+	if err != nil {
+		return err
+	}
+
+	c.clients.RBAC, err = rbacset.NewForConfig(c.kubeconfig)
+	if err != nil {
+		return err
+	}
+
+	c.clients.Kube, err = kubeset.NewForConfig(c.kubeconfig)
+	if err != nil {
+		return err
+	}
+
+	c.clients.Route, err = routesetv1.NewForConfig(c.kubeconfig)
+	if err != nil {
+		return err
+	}
+
+	c.clients.Config, err = configsetv1.NewForConfig(c.kubeconfig)
+	if err != nil {
+		return err
+	}
+
+	c.clients.RegOp, err = regopset.NewForConfig(c.kubeconfig)
 	if err != nil {
 		return err
 	}
@@ -340,17 +375,12 @@ func (c *Controller) Run(stopCh <-chan struct{}) error {
 		return err
 	}
 
-	regopClient, err := regopset.NewForConfig(c.kubeconfig)
-	if err != nil {
-		return err
-	}
-
 	configInformerFactory := configinformers.NewSharedInformerFactory(configClient, defaultResyncDuration)
-	kubeInformerFactory := kubeinformers.NewSharedInformerFactoryWithOptions(kubeClient, defaultResyncDuration, kubeinformers.WithNamespace(c.params.Deployment.Namespace))
-	openshiftConfigKubeInformerFactory := kubeinformers.NewSharedInformerFactoryWithOptions(kubeClient, defaultResyncDuration, kubeinformers.WithNamespace(openshiftConfigNamespace))
-	regopInformerFactory := regopinformers.NewSharedInformerFactory(regopClient, defaultResyncDuration)
+	kubeInformerFactory := kubeinformers.NewSharedInformerFactoryWithOptions(c.clients.Kube, defaultResyncDuration, kubeinformers.WithNamespace(c.params.Deployment.Namespace))
+	openshiftConfigKubeInformerFactory := kubeinformers.NewSharedInformerFactoryWithOptions(c.clients.Kube, defaultResyncDuration, kubeinformers.WithNamespace(openshiftConfigNamespace))
+	kubeSystemKubeInformerFactory := kubeinformers.NewSharedInformerFactoryWithOptions(c.clients.Kube, defaultResyncDuration, kubeinformers.WithNamespace(kubeSystemNamespace))
+	regopInformerFactory := regopinformers.NewSharedInformerFactory(c.clients.RegOp, defaultResyncDuration)
 	routeInformerFactory := routeinformers.NewSharedInformerFactoryWithOptions(routeClient, defaultResyncDuration, routeinformers.WithNamespace(c.params.Deployment.Namespace))
-	installerConfigInformerFactory := kubeinformers.NewSharedInformerFactoryWithOptions(kubeClient, defaultResyncDuration, kubeinformers.WithNamespace(installerConfigNamespace))
 
 	var informers []cache.SharedIndexInformer
 	for _, ctor := range []func() cache.SharedIndexInformer{
@@ -414,14 +444,24 @@ func (c *Controller) Run(stopCh <-chan struct{}) error {
 			c.listers.ClusterOperators = informer.Lister()
 			return informer.Informer()
 		},
+		func() cache.SharedIndexInformer {
+			informer := configInformerFactory.Config().V1().Proxies()
+			c.listers.ProxyConfigs = informer.Lister()
+			return informer.Informer()
+		},
 		func() cache.SharedIndexInformer {
 			informer := regopInformerFactory.Imageregistry().V1().Configs()
 			c.listers.RegistryConfigs = informer.Lister()
 			return informer.Informer()
 		},
 		func() cache.SharedIndexInformer {
-			informer := installerConfigInformerFactory.Core().V1().Secrets()
-			c.listers.InstallerSecrets = informer.Lister().Secrets(installerConfigNamespace)
+			informer := kubeSystemKubeInformerFactory.Core().V1().ConfigMaps()
+			c.listers.InstallerConfigMaps = informer.Lister().ConfigMaps(kubeSystemNamespace)
+			return informer.Informer()
+		},
+		func() cache.SharedIndexInformer {
+			informer := configInformerFactory.Config().V1().Infrastructures()
+			c.listers.Infrastructures = informer.Lister()
 			return informer.Informer()
 		},
 	} {
@@ -431,14 +471,13 @@ func (c *Controller) Run(stopCh <-chan struct{}) error {
 	}
 
 	configInformerFactory.Start(stopCh)
-	installerConfigInformerFactory.Start(stopCh)
 	kubeInformerFactory.Start(stopCh)
 	openshiftConfigKubeInformerFactory.Start(stopCh)
-	routeInformerFactory.Start(stopCh)
-	configInformerFactory.Start(stopCh)
+	kubeSystemKubeInformerFactory.Start(stopCh)
 	regopInformerFactory.Start(stopCh)
+	routeInformerFactory.Start(stopCh)
 
-	glog.Info("waiting for informer caches to sync")
+	klog.Info("waiting for informer caches to sync")
 	for _, informer := range informers {
 		if ok := cache.WaitForCacheSync(stopCh, informer.HasSynced); !ok {
 			return fmt.Errorf("failed to wait for caches to sync")
@@ -447,9 +486,9 @@ func (c *Controller) Run(stopCh <-chan struct{}) error {
 
 	go wait.Until(c.eventProcessor, time.Second, stopCh)
 
-	glog.Info("started events processor")
+	klog.Info("started events processor")
 	<-stopCh
-	glog.Info("shutting down events processor")
+	klog.Info("shutting down events processor")
 
 	return nil
 }
