@@ -2,22 +2,20 @@ package resource
 
 import (
 	"fmt"
-	"os"
 	"reflect"
 
-	"github.com/golang/glog"
-
+	appsapi "k8s.io/api/apps/v1"
 	kerrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	appslisters "k8s.io/client-go/listers/apps/v1"
+	"k8s.io/klog"
 
 	configapi "github.com/openshift/api/config/v1"
 	operatorapi "github.com/openshift/api/operator/v1"
 	configset "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
 	configlisters "github.com/openshift/client-go/config/listers/config/v1"
 	imageregistryv1 "github.com/openshift/cluster-image-registry-operator/pkg/apis/imageregistry/v1"
-	appsapi "k8s.io/api/apps/v1"
-	appslisters "k8s.io/client-go/listers/apps/v1"
 )
 
 var _ Mutator = &generatorClusterOperator{}
@@ -154,7 +152,7 @@ func (gco *generatorClusterOperator) syncConditions(op *configapi.ClusterOperato
 
 			newStatus, err := convertOperatorStatus(resourceCondition.Status)
 			if err != nil {
-				glog.Errorf("ignore condition of %s custom resource: %s", gco.cr.Name, err)
+				klog.Errorf("ignore condition of %s custom resource: %s", gco.cr.Name, err)
 				continue
 			}
 
@@ -182,7 +180,7 @@ func (gco *generatorClusterOperator) syncConditions(op *configapi.ClusterOperato
 		if !found {
 			conditionStatus, err := convertOperatorStatus(resourceCondition.Status)
 			if err != nil {
-				glog.Errorf("ignore condition of %s custom resource: %s", gco.cr.Name, err)
+				klog.Errorf("ignore condition of %s custom resource: %s", gco.cr.Name, err)
 				continue
 			}
 			conditions = append(conditions, configapi.ClusterOperatorStatusCondition{
@@ -213,49 +211,33 @@ func isDeploymentStatusAvailableAndUpdated(deploy *appsapi.Deployment) bool {
 		deploy.Status.UpdatedReplicas == deploy.Status.Replicas
 }
 
-// syncVersions updates reported version.
-//
-// If in "Managed" state we use the version stored as a annotation on registry'
-// Deployment, if not we use RELEASE_VERSION environment variable.
-func (gco *generatorClusterOperator) syncVersions(op *configapi.ClusterOperator) (bool, error) {
-	if gco.cr == nil {
-		return false, fmt.Errorf("invalid nil configuration provided")
+func (gco *generatorClusterOperator) syncVersions(op *configapi.ClusterOperator) (modified bool, err error) {
+	deploy, err := gco.deployLister.Get(imageregistryv1.ImageRegistryName)
+	if err != nil {
+		if kerrors.IsNotFound(err) {
+			err = nil
+		}
+		return
 	}
 
-	version := os.Getenv("RELEASE_VERSION")
-	if gco.cr.Spec.ManagementState == operatorapi.Managed {
-		deploy, err := gco.deployLister.Get(imageregistryv1.ImageRegistryName)
-		if err != nil {
-			if kerrors.IsNotFound(err) {
-				return false, nil
-			}
-			return false, err
-		}
-
-		if !isDeploymentStatusAvailableAndUpdated(deploy) {
-			return false, nil
-		}
-
-		version = deploy.Annotations[imageregistryv1.VersionAnnotation]
-	}
-
-	if len(version) == 0 {
-		return false, nil
+	deploymentVersion := deploy.Annotations[imageregistryv1.VersionAnnotation]
+	if len(deploymentVersion) == 0 || !isDeploymentStatusAvailableAndUpdated(deploy) {
+		return
 	}
 
 	newVersions := []configapi.OperandVersion{
 		{
 			Name:    "operator",
-			Version: version,
+			Version: deploymentVersion,
 		},
 	}
 
-	if reflect.DeepEqual(op.Status.Versions, newVersions) {
-		return false, nil
+	if !reflect.DeepEqual(op.Status.Versions, newVersions) {
+		op.Status.Versions = newVersions
+		modified = true
 	}
 
-	op.Status.Versions = newVersions
-	return true, nil
+	return
 }
 
 func (gco *generatorClusterOperator) syncRelatedObjects(op *configapi.ClusterOperator) (modified bool) {
