@@ -7,12 +7,15 @@ import (
 	"strings"
 
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
 	coreset "k8s.io/client-go/kubernetes/typed/core/v1"
 
-	"github.com/openshift/cluster-image-registry-operator/pkg/apis/imageregistry/v1"
+	configapiv1 "github.com/openshift/api/config/v1"
+	configlisters "github.com/openshift/client-go/config/listers/config/v1"
+	v1 "github.com/openshift/cluster-image-registry-operator/pkg/apis/imageregistry/v1"
 	"github.com/openshift/cluster-image-registry-operator/pkg/parameters"
 	"github.com/openshift/cluster-image-registry-operator/pkg/storage"
 )
@@ -93,7 +96,7 @@ func storageConfigure(driver storage.Driver) (envs []corev1.EnvVar, volumes []co
 	return
 }
 
-func makePodTemplateSpec(coreClient coreset.CoreV1Interface, driver storage.Driver, params *parameters.Globals, cr *v1.Config) (corev1.PodTemplateSpec, *dependencies, error) {
+func makePodTemplateSpec(coreClient coreset.CoreV1Interface, proxyLister configlisters.ProxyLister, driver storage.Driver, params *parameters.Globals, cr *v1.Config) (corev1.PodTemplateSpec, *dependencies, error) {
 	env, volumes, mounts, err := storageConfigure(driver)
 	if err != nil {
 		return corev1.PodTemplateSpec{}, nil, err
@@ -112,6 +115,14 @@ func makePodTemplateSpec(coreClient coreset.CoreV1Interface, driver storage.Driv
 		}
 	}
 
+	clusterProxy, err := proxyLister.Get(v1.ClusterProxyResourceName)
+	if errors.IsNotFound(err) {
+		clusterProxy = &configapiv1.Proxy{}
+	} else if err != nil {
+		// TODO: should we report Degraded?
+		return corev1.PodTemplateSpec{}, deps, fmt.Errorf("unable to get cluster proxy configuration: %v", err)
+	}
+
 	env = append(env,
 		corev1.EnvVar{Name: "REGISTRY_HTTP_ADDR", Value: fmt.Sprintf(":%d", params.Container.Port)},
 		corev1.EnvVar{Name: "REGISTRY_HTTP_NET", Value: "tcp"},
@@ -131,14 +142,20 @@ func makePodTemplateSpec(coreClient coreset.CoreV1Interface, driver storage.Driv
 
 	if cr.Spec.Proxy.HTTP != "" {
 		env = append(env, corev1.EnvVar{Name: "HTTP_PROXY", Value: cr.Spec.Proxy.HTTP})
+	} else if clusterProxy.Status.HTTPProxy != "" {
+		env = append(env, corev1.EnvVar{Name: "HTTP_PROXY", Value: clusterProxy.Status.HTTPProxy})
 	}
 
 	if cr.Spec.Proxy.HTTPS != "" {
 		env = append(env, corev1.EnvVar{Name: "HTTPS_PROXY", Value: cr.Spec.Proxy.HTTPS})
+	} else if clusterProxy.Status.HTTPSProxy != "" {
+		env = append(env, corev1.EnvVar{Name: "HTTPS_PROXY", Value: clusterProxy.Status.HTTPSProxy})
 	}
 
 	if cr.Spec.Proxy.NoProxy != "" {
 		env = append(env, corev1.EnvVar{Name: "NO_PROXY", Value: cr.Spec.Proxy.NoProxy})
+	} else if clusterProxy.Status.NoProxy != "" {
+		env = append(env, corev1.EnvVar{Name: "NO_PROXY", Value: clusterProxy.Status.NoProxy})
 	}
 
 	if cr.Spec.Requests.Read.MaxRunning != 0 || cr.Spec.Requests.Read.MaxInQueue != 0 {
@@ -199,7 +216,7 @@ func makePodTemplateSpec(coreClient coreset.CoreV1Interface, driver storage.Driv
 		corev1.EnvVar{Name: "REGISTRY_HTTP_TLS_KEY", Value: "/etc/secrets/tls.key"},
 	)
 
-	// Certificates
+	// Registry certificate authorities - mount as high-priority trust source anchors
 	vol = corev1.Volume{
 		Name: "registry-certificates",
 		VolumeSource: corev1.VolumeSource{
@@ -212,7 +229,39 @@ func makePodTemplateSpec(coreClient coreset.CoreV1Interface, driver storage.Driv
 	}
 	volumes = append(volumes, vol)
 	mounts = append(mounts, corev1.VolumeMount{Name: vol.Name, MountPath: "/etc/pki/ca-trust/source/anchors"})
-	deps.AddConfigMap(vol.VolumeSource.ConfigMap.LocalObjectReference.Name)
+	deps.AddConfigMap(v1.ImageRegistryCertificatesName)
+
+	// Cluster trusted certificate authorities - mount to /usr/share/pki/ca-trust-source/ to add
+	// CAs as low-priority trust sources. Registry runs update-ca-trust extract on startup, which
+	// merges the registry CAs with the cluster's trusted CAs into a single CA bundle.
+	//
+	// See man update-ca-trust for more information.
+	optional := true
+	vol = corev1.Volume{
+		Name: "trusted-ca",
+		VolumeSource: corev1.VolumeSource{
+			ConfigMap: &corev1.ConfigMapVolumeSource{
+				LocalObjectReference: corev1.LocalObjectReference{
+					Name: params.TrustedCA.Name,
+				},
+				// Trust bundle is in PEM format - needs to be mounted to /anchors so that
+				// update-ca-trust extract knows that these CAs should always be trusted.
+				// This also ensures that no other low-priority trust is present in the container.
+				//
+				// See man update-ca-trust for more information.
+				Items: []corev1.KeyToPath{
+					{
+						Key:  "ca-bundle.crt",
+						Path: "anchors/ca-bundle.crt",
+					},
+				},
+				Optional: &optional,
+			},
+		},
+	}
+	volumes = append(volumes, vol)
+	mounts = append(mounts, corev1.VolumeMount{Name: vol.Name, MountPath: "/usr/share/pki/ca-trust-source"})
+	deps.AddConfigMap(params.TrustedCA.Name)
 
 	image := os.Getenv("IMAGE")
 
