@@ -3,8 +3,9 @@ package s3
 import (
 	"context"
 	"fmt"
+	"net/http"
+	"net/url"
 	"reflect"
-	"strings"
 
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/aws/awserr"
@@ -13,58 +14,121 @@ import (
 	"github.com/aws/aws-sdk-go/aws/session"
 	"github.com/aws/aws-sdk-go/service/s3"
 	"github.com/aws/aws-sdk-go/service/s3/s3manager"
-
-	operatorapi "github.com/openshift/api/operator/v1"
+	"golang.org/x/net/http/httpproxy"
 
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/errors"
+	"k8s.io/client-go/rest"
 
-	"k8s.io/apimachinery/pkg/util/uuid"
-
+	configapiv1 "github.com/openshift/api/config/v1"
+	operatorapi "github.com/openshift/api/operator/v1"
 	imageregistryv1 "github.com/openshift/cluster-image-registry-operator/pkg/apis/imageregistry/v1"
 	regopclient "github.com/openshift/cluster-image-registry-operator/pkg/client"
-	"github.com/openshift/cluster-image-registry-operator/pkg/clusterconfig"
 	"github.com/openshift/cluster-image-registry-operator/pkg/storage/util"
 	"github.com/openshift/cluster-image-registry-operator/version"
 )
 
-var (
-	s3Service *s3.S3
-)
+type S3 struct {
+	AccessKey string
+	SecretKey string
+	Bucket    string
+	Region    string
+}
 
 type driver struct {
-	Config  *imageregistryv1.ImageRegistryConfigStorageS3
-	Listers *regopclient.Listers
+	Context    context.Context
+	Config     *imageregistryv1.ImageRegistryConfigStorageS3
+	KubeConfig *rest.Config
+	Listers    *regopclient.Listers
 }
 
 // NewDriver creates a new s3 storage driver
 // Used during bootstrapping
-func NewDriver(c *imageregistryv1.ImageRegistryConfigStorageS3, listers *regopclient.Listers) *driver {
+func NewDriver(ctx context.Context, c *imageregistryv1.ImageRegistryConfigStorageS3, kubeconfig *rest.Config, listers *regopclient.Listers) *driver {
 	return &driver{
-		Config:  c,
-		Listers: listers,
+		Context:    ctx,
+		Config:     c,
+		KubeConfig: kubeconfig,
+		Listers:    listers,
 	}
 }
 
+// GetConfig reads configuration for the S3 cloud platform services.
+func GetConfig(kubeconfig *rest.Config, listers *regopclient.Listers) (*S3, error) {
+	cfg := &S3{}
+
+	infra, err := util.GetInfrastructure(listers)
+	if err != nil {
+		return nil, err
+	}
+
+	if infra.Status.PlatformStatus != nil && infra.Status.PlatformStatus.Type == configapiv1.AWSPlatformType {
+		cfg.Region = infra.Status.PlatformStatus.AWS.Region
+	}
+
+	// Look for a user defined secret to get the AWS credentials from first
+	sec, err := listers.Secrets.Get(imageregistryv1.ImageRegistryPrivateConfigurationUser)
+	if err != nil && errors.IsNotFound(err) {
+		// Fall back to those provided by the credential minter if nothing is provided by the user
+		sec, err = listers.Secrets.Get(imageregistryv1.CloudCredentialsName)
+		if err != nil {
+			return nil, fmt.Errorf("unable to get cluster minted credentials %q: %v", fmt.Sprintf("%s/%s", imageregistryv1.ImageRegistryOperatorNamespace, imageregistryv1.CloudCredentialsName), err)
+		}
+
+		if v, ok := sec.Data["aws_access_key_id"]; ok {
+			cfg.AccessKey = string(v)
+		} else {
+			return nil, fmt.Errorf("secret %q does not contain required key \"aws_access_key_id\"", fmt.Sprintf("%s/%s", imageregistryv1.ImageRegistryOperatorNamespace, imageregistryv1.CloudCredentialsName))
+		}
+		if v, ok := sec.Data["aws_secret_access_key"]; ok {
+			cfg.SecretKey = string(v)
+		} else {
+			return nil, fmt.Errorf("secret %q does not contain required key \"aws_secret_access_key\"", fmt.Sprintf("%s/%s", imageregistryv1.ImageRegistryOperatorNamespace, imageregistryv1.CloudCredentialsName))
+		}
+	} else if err != nil {
+		return nil, err
+	} else {
+		if v, ok := sec.Data["REGISTRY_STORAGE_S3_ACCESSKEY"]; ok {
+			cfg.AccessKey = string(v)
+		} else {
+			return nil, fmt.Errorf("secret %q does not contain required key \"REGISTRY_STORAGE_S3_ACCESSKEY\"", fmt.Sprintf("%s/%s", imageregistryv1.ImageRegistryOperatorNamespace, imageregistryv1.ImageRegistryPrivateConfigurationUser))
+		}
+		if v, ok := sec.Data["REGISTRY_STORAGE_S3_SECRETKEY"]; ok {
+			cfg.SecretKey = string(v)
+		} else {
+			return nil, fmt.Errorf("secret %q does not contain required key \"REGISTRY_STORAGE_S3_SECRETKEY\"", fmt.Sprintf("%s/%s", imageregistryv1.ImageRegistryOperatorNamespace, imageregistryv1.ImageRegistryPrivateConfigurationUser))
+
+		}
+	}
+
+	return cfg, nil
+}
+
 // getS3Service returns a client that allows us to interact
 // with the aws S3 service
 func (d *driver) getS3Service() (*s3.S3, error) {
-	if s3Service != nil {
-		return s3Service, nil
-	}
-
-	cfg, err := clusterconfig.GetAWSConfig(d.Listers)
+	cfg, err := GetConfig(d.KubeConfig, d.Listers)
 	if err != nil {
 		return nil, err
 	}
 
 	if len(d.Config.Region) == 0 {
-		d.Config.Region = cfg.Storage.S3.Region
+		d.Config.Region = cfg.Region
 	}
 
+	// A custom HTTPClient is used here since the default HTTPClients ProxyFromEnvironment
+	// uses a cache which won't let us update the proxy env vars
 	sess, err := session.NewSession(&aws.Config{
-		Credentials: credentials.NewStaticCredentials(cfg.Storage.S3.AccessKey, cfg.Storage.S3.SecretKey, ""),
+		Credentials: credentials.NewStaticCredentials(cfg.AccessKey, cfg.SecretKey, ""),
 		Region:      &d.Config.Region,
 		Endpoint:    &d.Config.RegionEndpoint,
+		HTTPClient: &http.Client{
+			Transport: &http.Transport{
+				Proxy: func(req *http.Request) (*url.URL, error) {
+					return httpproxy.FromEnvironment().ProxyFunc()(req.URL)
+				},
+			},
+		},
 	})
 	if err != nil {
 		return nil, err
@@ -74,9 +138,7 @@ func (d *driver) getS3Service() (*s3.S3, error) {
 		Fn:   request.MakeAddToUserAgentHandler("openshift.io cluster-image-registry-operator", version.Version),
 	})
 
-	s3Service := s3.New(sess)
-
-	return s3Service, nil
+	return s3.New(sess), nil
 
 }
 
@@ -185,14 +247,14 @@ func (d *driver) Volumes() ([]corev1.Volume, []corev1.VolumeMount, error) {
 
 // Secrets returns a map of the storage access secrets.
 func (d *driver) Secrets() (map[string]string, error) {
-	cfg, err := clusterconfig.GetAWSConfig(d.Listers)
+	cfg, err := GetConfig(d.KubeConfig, d.Listers)
 	if err != nil {
 		return nil, err
 	}
 
 	return map[string]string{
-		"REGISTRY_STORAGE_S3_ACCESSKEY": cfg.Storage.S3.AccessKey,
-		"REGISTRY_STORAGE_S3_SECRETKEY": cfg.Storage.S3.SecretKey,
+		"REGISTRY_STORAGE_S3_ACCESSKEY": cfg.AccessKey,
+		"REGISTRY_STORAGE_S3_SECRETKEY": cfg.SecretKey,
 	}, nil
 }
 
@@ -207,7 +269,7 @@ func (d *driver) bucketExists(bucketName string) error {
 		return err
 	}
 
-	_, err = svc.HeadBucket(&s3.HeadBucketInput{
+	_, err = svc.HeadBucketWithContext(d.Context, &s3.HeadBucketInput{
 		Bucket: aws.String(bucketName),
 	})
 
@@ -258,12 +320,7 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 		return err
 	}
 
-	ic, err := clusterconfig.GetInstallConfig()
-	if err != nil {
-		return err
-	}
-
-	cv, err := util.GetClusterVersionConfig()
+	infra, err := util.GetInfrastructure(d.Listers)
 	if err != nil {
 		return err
 	}
@@ -298,14 +355,18 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 
 	} else {
 		generatedName := false
-		for i := 0; i < 5000; i++ {
+		// Retry up to 5000 times if we get a naming conflict
+		const numRetries = 5000
+		for i := 0; i < numRetries; i++ {
 			// If the bucket name is blank, let's generate one
 			if len(d.Config.Bucket) == 0 {
-				d.Config.Bucket = fmt.Sprintf("%s-%s-%s-%s", imageregistryv1.ImageRegistryName, d.Config.Region, strings.Replace(string(cv.Spec.ClusterID), "-", "", -1), strings.Replace(string(uuid.NewUUID()), "-", "", -1))[0:62]
+				if d.Config.Bucket, err = util.GenerateStorageName(d.Listers, d.Config.Region); err != nil {
+					return err
+				}
 				generatedName = true
 			}
 
-			_, err := svc.CreateBucket(&s3.CreateBucketInput{
+			_, err := svc.CreateBucketWithContext(d.Context, &s3.CreateBucketInput{
 				Bucket: aws.String(d.Config.Bucket),
 			})
 			if err != nil {
@@ -340,7 +401,7 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 	}
 
 	// Wait until the bucket exists
-	if err := svc.WaitUntilBucketExists(&s3.HeadBucketInput{
+	if err := svc.WaitUntilBucketExistsWithContext(d.Context, &s3.HeadBucketInput{
 		Bucket: aws.String(d.Config.Bucket),
 	}); err != nil {
 		if aerr, ok := err.(awserr.Error); ok {
@@ -350,31 +411,58 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 		return err
 	}
 
+	// Block public access to the s3 bucket and its objects by default
+	if cr.Status.StorageManaged {
+		_, err := svc.PutPublicAccessBlockWithContext(d.Context, &s3.PutPublicAccessBlockInput{
+			Bucket: aws.String(d.Config.Bucket),
+			PublicAccessBlockConfiguration: &s3.PublicAccessBlockConfiguration{
+				BlockPublicAcls:       aws.Bool(true),
+				BlockPublicPolicy:     aws.Bool(true),
+				IgnorePublicAcls:      aws.Bool(true),
+				RestrictPublicBuckets: aws.Bool(true),
+			},
+		})
+
+		if err != nil {
+			if aerr, ok := err.(awserr.Error); ok {
+				util.UpdateCondition(cr, imageregistryv1.StoragePublicAccessBlocked, operatorapi.ConditionFalse, aerr.Code(), aerr.Error())
+			} else {
+				util.UpdateCondition(cr, imageregistryv1.StoragePublicAccessBlocked, operatorapi.ConditionFalse, "Unknown Error Occurred", err.Error())
+			}
+		} else {
+			util.UpdateCondition(cr, imageregistryv1.StoragePublicAccessBlocked, operatorapi.ConditionTrue, "Public Access Block Successful", "Public access to the S3 bucket and its contents have been successfully blocked.")
+			cr.Status.Storage.S3 = d.Config.DeepCopy()
+			cr.Spec.Storage.S3 = d.Config.DeepCopy()
+		}
+	}
+
 	// Tag the bucket with the openshiftClusterID
 	// along with any user defined tags from the cluster configuration
 	if cr.Status.StorageManaged {
-		if ic.Platform.AWS != nil {
-			var tagSet []*s3.Tag
-			tagSet = append(tagSet, &s3.Tag{Key: aws.String("openshiftClusterID"), Value: aws.String(string(cv.Spec.ClusterID))})
-			for k, v := range ic.Platform.AWS.UserTags {
-				tagSet = append(tagSet, &s3.Tag{Key: aws.String(k), Value: aws.String(v)})
-			}
+		_, err := svc.PutBucketTaggingWithContext(d.Context, &s3.PutBucketTaggingInput{
+			Bucket: aws.String(d.Config.Bucket),
+			Tagging: &s3.Tagging{
 
-			_, err := svc.PutBucketTagging(&s3.PutBucketTaggingInput{
-				Bucket: aws.String(d.Config.Bucket),
-				Tagging: &s3.Tagging{
-					TagSet: tagSet,
+				TagSet: []*s3.Tag{
+					{
+						Key:   aws.String("kubernetes.io/cluster/" + infra.Status.InfrastructureName),
+						Value: aws.String("owned"),
+					},
+					{
+						Key:   aws.String("Name"),
+						Value: aws.String(infra.Status.InfrastructureName + "-image-registry"),
+					},
 				},
-			})
-			if err != nil {
-				if aerr, ok := err.(awserr.Error); ok {
-					util.UpdateCondition(cr, imageregistryv1.StorageTagged, operatorapi.ConditionFalse, aerr.Code(), aerr.Error())
-				} else {
-					util.UpdateCondition(cr, imageregistryv1.StorageTagged, operatorapi.ConditionFalse, "Unknown Error Occurred", err.Error())
-				}
+			},
+		})
+		if err != nil {
+			if aerr, ok := err.(awserr.Error); ok {
+				util.UpdateCondition(cr, imageregistryv1.StorageTagged, operatorapi.ConditionFalse, aerr.Code(), aerr.Error())
 			} else {
-				util.UpdateCondition(cr, imageregistryv1.StorageTagged, operatorapi.ConditionTrue, "Tagging Successful", "UserTags were successfully applied to the S3 bucket")
+				util.UpdateCondition(cr, imageregistryv1.StorageTagged, operatorapi.ConditionFalse, "Unknown Error Occurred", err.Error())
 			}
+		} else {
+			util.UpdateCondition(cr, imageregistryv1.StorageTagged, operatorapi.ConditionTrue, "Tagging Successful", "Tags were successfully applied to the S3 bucket")
 		}
 	}
 
@@ -396,7 +484,7 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 			encryptionType = s3.ServerSideEncryptionAes256
 		}
 
-		_, err = svc.PutBucketEncryption(&s3.PutBucketEncryptionInput{
+		_, err = svc.PutBucketEncryptionWithContext(d.Context, &s3.PutBucketEncryptionInput{
 			Bucket: aws.String(d.Config.Bucket),
 			ServerSideEncryptionConfiguration: &s3.ServerSideEncryptionConfiguration{
 				Rules: []*s3.ServerSideEncryptionRule{
@@ -426,7 +514,7 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 
 	// Enable default incomplete multipart upload cleanup after one (1) day
 	if cr.Status.StorageManaged {
-		_, err = svc.PutBucketLifecycleConfiguration(&s3.PutBucketLifecycleConfigurationInput{
+		_, err = svc.PutBucketLifecycleConfigurationWithContext(d.Context, &s3.PutBucketLifecycleConfigurationInput{
 			Bucket: aws.String(d.Config.Bucket),
 			LifecycleConfiguration: &s3.BucketLifecycleConfiguration{
 				Rules: []*s3.LifecycleRule{
@@ -458,6 +546,7 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 }
 
 // RemoveStorage deletes the storage medium that we created
+// The s3 bucket must be empty before it can be removed
 func (d *driver) RemoveStorage(cr *imageregistryv1.Config) (bool, error) {
 	if !cr.Status.StorageManaged || len(d.Config.Bucket) == 0 {
 		return false, nil
@@ -472,12 +561,12 @@ func (d *driver) RemoveStorage(cr *imageregistryv1.Config) (bool, error) {
 		Bucket: aws.String(d.Config.Bucket),
 	})
 
-	err = s3manager.NewBatchDeleteWithClient(svc).Delete(context.Background(), iter)
+	err = s3manager.NewBatchDeleteWithClient(svc).Delete(d.Context, iter)
 	if err != nil && !isBucketNotFound(err) {
 		return false, err
 	}
 
-	_, err = svc.DeleteBucket(&s3.DeleteBucketInput{
+	_, err = svc.DeleteBucketWithContext(d.Context, &s3.DeleteBucketInput{
 		Bucket: aws.String(d.Config.Bucket),
 	})
 
@@ -494,7 +583,7 @@ func (d *driver) RemoveStorage(cr *imageregistryv1.Config) (bool, error) {
 	}
 
 	// Wait until the bucket does not exist
-	if err := svc.WaitUntilBucketNotExists(&s3.HeadBucketInput{
+	if err := svc.WaitUntilBucketNotExistsWithContext(d.Context, &s3.HeadBucketInput{
 		Bucket: aws.String(d.Config.Bucket),
 	}); err != nil {
 		if aerr, ok := err.(awserr.Error); ok {
@@ -517,25 +606,4 @@ func (d *driver) RemoveStorage(cr *imageregistryv1.Config) (bool, error) {
 	util.UpdateCondition(cr, imageregistryv1.StorageExists, operatorapi.ConditionFalse, "S3 Bucket Deleted", "The S3 bucket has been removed.")
 
 	return false, nil
-
-}
-
-func (d *driver) CompleteConfiguration(cr *imageregistryv1.Config) error {
-	cfg, err := clusterconfig.GetAWSConfig(d.Listers)
-	if err != nil {
-		return err
-	}
-
-	if len(d.Config.Region) == 0 {
-		d.Config.Region = cfg.Storage.S3.Region
-	}
-	if cr.Spec.Storage.S3 == nil {
-		cr.Spec.Storage.S3 = &imageregistryv1.ImageRegistryConfigStorageS3{}
-	}
-	if cr.Status.Storage.S3 == nil {
-		cr.Status.Storage.S3 = &imageregistryv1.ImageRegistryConfigStorageS3{}
-	}
-	cr.Spec.Storage.S3 = d.Config.DeepCopy()
-
-	return nil
 }
