@@ -19,10 +19,10 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/wait"
 
+	configapiv1 "github.com/openshift/api/config/v1"
 	operatorapiv1 "github.com/openshift/api/operator/v1"
 
 	imageregistryapiv1 "github.com/openshift/cluster-image-registry-operator/pkg/apis/imageregistry/v1"
-	"github.com/openshift/cluster-image-registry-operator/pkg/storage/util"
 	"github.com/openshift/cluster-image-registry-operator/test/framework"
 )
 
@@ -174,6 +174,188 @@ func TestRouteConfiguration(t *testing.T) {
 	framework.EnsureExternalRoutesExist(t, client, []string{hostname})
 }
 
+func TestOperatorProxyConfiguration(t *testing.T) {
+	client := framework.MustNewClientset(t, nil)
+
+	defer framework.MustRemoveImageRegistry(t, client)
+	defer framework.ResetClusterProxyConfig(client)
+
+	framework.MustDeployImageRegistry(t, client, nil)
+	framework.MustEnsureImageRegistryIsAvailable(t, client)
+	framework.MustEnsureClusterOperatorStatusIsNormal(t, client)
+
+	// Wait for the registry operator to be deployed
+	deployment, err := framework.WaitForRegistryOperatorDeployment(client)
+	if err != nil {
+		framework.DumpImageRegistryResource(t, client)
+		framework.DumpOperatorLogs(t, client)
+		t.Fatal(err)
+	}
+
+	// Get the service network to set as NO_PROXY so that the
+	// operator will come up once it is re-deployed
+	network, err := client.Networks().Get("cluster", metav1.GetOptions{})
+	if err != nil {
+		t.Fatalf("unable to get network configuration: %v", err)
+	}
+
+	// Set the proxy env vars
+	if _, err := client.Deployments(framework.OperatorDeploymentNamespace).Patch(framework.OperatorDeploymentName, types.StrategicMergePatchType, []byte(fmt.Sprintf(`{"spec": {"template": {"spec": {"containers": [{"name":"cluster-image-registry-operator","env":[{"name":"HTTP_PROXY","value":"http://http.example.org"},{"name":"HTTPS_PROXY","value":"https://https.example.org"},{"name":"NO_PROXY","value":"%s"}]}]}}}}`, strings.Join(network.Spec.ServiceNetwork, ",")))); err != nil {
+		t.Fatalf("failed to patch operator env vars: %v", err)
+	}
+	defer func() {
+		if _, err := client.Deployments(framework.OperatorDeploymentNamespace).Patch(framework.OperatorDeploymentName, types.StrategicMergePatchType, []byte(`{"spec": {"template": {"spec": {"containers": [{"name":"cluster-image-registry-operator","env":[{"name":"HTTP_PROXY","value":""},{"name":"HTTPS_PROXY","value":""},{"name":"NO_PROXY","value":""}]}]}}}}`)); err != nil {
+			t.Fatalf("failed to patch operator env vars: %v", err)
+		}
+	}()
+
+	// Wait for the registry operator to be re-deployed
+	// after the proxy information is injected into the deployment
+	_, err = framework.WaitForNewRegistryOperatorDeployment(client, deployment.Status.ObservedGeneration)
+	if err != nil {
+		framework.DumpImageRegistryResource(t, client)
+		framework.DumpOperatorLogs(t, client)
+		framework.DumpOperatorDeployment(t, client)
+		t.Fatal(err)
+	}
+
+	// Wait for the image registry resource to have an updated StorageExists condition
+	// showing that the operator can no longer reach the storage providers api
+	errs := framework.ConditionExistsWithStatusAndReason(client, imageregistryapiv1.StorageExists, operatorapiv1.ConditionUnknown, "Unknown Error Occurred")
+	if len(errs) != 0 {
+		framework.DumpImageRegistryResource(t, client)
+		framework.DumpOperatorLogs(t, client)
+		framework.DumpOperatorDeployment(t, client)
+
+		for _, err := range errs {
+			t.Errorf("%#v", err)
+		}
+	}
+
+	if _, err := client.Deployments(framework.OperatorDeploymentNamespace).Patch(framework.OperatorDeploymentName, types.StrategicMergePatchType, []byte(`{"spec": {"template": {"spec": {"containers": [{"name":"cluster-image-registry-operator","env":[{"name":"HTTP_PROXY","value":""},{"name":"HTTPS_PROXY","value":""},{"name":"NO_PROXY","value":""}]}]}}}}`)); err != nil {
+		t.Fatalf("failed to patch operator env vars: %v", err)
+	}
+
+	// Wait for the image registry resource to have an updated StorageExists condition
+	// showing that operator can now reach the storage providers api
+	errs = framework.ConditionExistsWithStatusAndReason(client, imageregistryapiv1.StorageExists, operatorapiv1.ConditionTrue, "")
+	if len(errs) != 0 {
+		for _, err := range errs {
+			t.Errorf("%#v", err)
+		}
+	}
+}
+
+func TestOperandProxyConfiguration(t *testing.T) {
+	client := framework.MustNewClientset(t, nil)
+
+	defer framework.MustRemoveImageRegistry(t, client)
+	defer framework.ResetClusterProxyConfig(client)
+	defer framework.ResetResourceProxyConfig(client)
+
+	resourceProxyConfig := imageregistryapiv1.ImageRegistryConfigProxy{
+		NoProxy: "resourcenoproxy.example.com",
+		HTTP:    "http://resourcehttpproxy.example.com",
+		HTTPS:   "https://resourcehttpsproxy.example.com",
+	}
+
+	clusterProxyConfig := configapiv1.ProxySpec{
+		NoProxy:    "clusternoproxy.example.com",
+		HTTPProxy:  "http://clusterhttpproxy.example.com",
+		HTTPSProxy: "https://clusterhttpsproxy.example.com",
+	}
+
+	proxyEnvVars := map[string][]corev1.EnvVar{
+		"emtpyVars": {
+			{Name: "NO_PROXY", Value: "", ValueFrom: nil},
+			{Name: "HTTP_PROXY", Value: "", ValueFrom: nil},
+			{Name: "HTTPS_PROXY", Value: "", ValueFrom: nil},
+		},
+		"resourceVars": {
+			{Name: "NO_PROXY", Value: resourceProxyConfig.NoProxy, ValueFrom: nil},
+			{Name: "HTTP_PROXY", Value: resourceProxyConfig.HTTP, ValueFrom: nil},
+			{Name: "HTTPS_PROXY", Value: resourceProxyConfig.HTTPS, ValueFrom: nil},
+		},
+		"clusterVars": {
+			{Name: "NO_PROXY", Value: clusterProxyConfig.NoProxy, ValueFrom: nil},
+			{Name: "HTTP_PROXY", Value: clusterProxyConfig.HTTPProxy, ValueFrom: nil},
+			{Name: "HTTPS_PROXY", Value: clusterProxyConfig.HTTPSProxy, ValueFrom: nil},
+		},
+	}
+
+	cr := &imageregistryapiv1.Config{
+		TypeMeta: metav1.TypeMeta{
+			APIVersion: imageregistryapiv1.SchemeGroupVersion.String(),
+			Kind:       "Config",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name: imageregistryapiv1.ImageRegistryResourceName,
+		},
+		Spec: imageregistryapiv1.ImageRegistrySpec{
+			ManagementState: operatorapiv1.Managed,
+			Storage: imageregistryapiv1.ImageRegistryConfigStorage{
+				EmptyDir: &imageregistryapiv1.ImageRegistryConfigStorageEmptyDir{},
+			},
+			Replicas: 1,
+		},
+	}
+	framework.MustDeployImageRegistry(t, client, cr)
+	framework.MustEnsureImageRegistryIsAvailable(t, client)
+	framework.MustEnsureClusterOperatorStatusIsNormal(t, client)
+
+	t.Logf("waiting for the operator to recreate the deployment...")
+	registryDeployment, err := framework.WaitForRegistryDeployment(client)
+	if err != nil {
+		framework.DumpImageRegistryResource(t, client)
+		framework.DumpOperatorLogs(t, client)
+		t.Fatal(err)
+	}
+
+	// Check that the default deployment does not contain any proxy settings
+	for _, err = range framework.CheckEnvVars(proxyEnvVars["emptyVars"], registryDeployment.Spec.Template.Spec.Containers[0].Env, true) {
+		t.Errorf("%v", err)
+	}
+
+	// Patch the cluster proxy config to set the proxy settings
+	if err := framework.SetClusterProxyConfig(clusterProxyConfig, client); err != nil {
+		t.Errorf("unable to patch cluster proxy instance: %v", err)
+	}
+
+	t.Logf("waiting for the operator to recreate the deployment...")
+	registryDeployment, err = framework.WaitForNewRegistryDeployment(client, registryDeployment.Status.ObservedGeneration)
+	if err != nil {
+		framework.DumpImageRegistryResource(t, client)
+		framework.DumpOperatorLogs(t, client)
+		t.Fatal(err)
+	}
+
+	// Check that the new deployment contains the cluster proxy settings
+	for _, err = range framework.CheckEnvVars(proxyEnvVars["clusterVars"], registryDeployment.Spec.Template.Spec.Containers[0].Env, true) {
+		t.Errorf("%v", err)
+		framework.DumpClusterProxyResource(t, client)
+		framework.DumpImageRegistryDeployment(t, client)
+		framework.DumpOperatorDeployment(t, client)
+	}
+
+	// Patch the image registry resource to contain the proxy settings
+	if err := framework.SetResourceProxyConfig(resourceProxyConfig, client); err != nil {
+		t.Errorf("unable to set resource proxy configuration: %v", err)
+	}
+
+	t.Logf("waiting for the operator to recreate the deployment...")
+	registryDeployment, err = framework.WaitForNewRegistryDeployment(client, registryDeployment.Status.ObservedGeneration)
+	if err != nil {
+		framework.DumpImageRegistryResource(t, client)
+		framework.DumpOperatorLogs(t, client)
+		t.Fatal(err)
+	}
+
+	// Check that the new deployment contains the resource proxy settings (overriding the cluster proxy config)
+	for _, err = range framework.CheckEnvVars(proxyEnvVars["resourceVars"], registryDeployment.Spec.Template.Spec.Containers[0].Env, true) {
+		t.Errorf("%v", err)
+	}
+}
+
 func generateCertificate(hostname string) ([]byte, []byte, error) {
 	priv, err := rsa.GenerateKey(rand.Reader, 2048)
 	if err != nil {
@@ -229,7 +411,7 @@ func TestSecureRouteConfiguration(t *testing.T) {
 		"tls.key": string(key),
 	}
 
-	if _, err := util.CreateOrUpdateSecret(tlsSecretName, imageregistryapiv1.ImageRegistryOperatorNamespace, tlsSecretData); err != nil {
+	if _, err := framework.CreateOrUpdateSecret(tlsSecretName, imageregistryapiv1.ImageRegistryOperatorNamespace, tlsSecretData); err != nil {
 		t.Fatalf("unable to create secret: %s", err)
 	}
 
@@ -378,7 +560,7 @@ func TestRequests(t *testing.T) {
 		{Name: "REGISTRY_OPENSHIFT_REQUESTS_WRITE_MAXWAITINQUEUE", Value: "6h0m0s", ValueFrom: nil},
 	}
 
-	for _, err = range framework.CheckEnvVars(expectedEnvVars, deploy.Spec.Template.Spec.Containers[0].Env) {
+	for _, err = range framework.CheckEnvVars(expectedEnvVars, deploy.Spec.Template.Spec.Containers[0].Env, false) {
 		t.Errorf("%v", err)
 	}
 }
